@@ -13,22 +13,22 @@
 #include <iostream>
 #include <list>
 
-#include "zypp/base/Logger.h"
-#include "zypp/ExternalProgram.h"
-#include "zypp/base/String.h"
-#include "zypp/base/Gettext.h"
-#include "zypp/base/Sysconfig.h"
-#include "zypp/base/Gettext.h"
+#include <zypp/base/Logger.h>
+#include <zypp/ExternalProgram.h>
+#include <zypp/base/String.h>
+#include <zypp/base/Gettext.h>
+#include <zypp/base/Sysconfig.h>
+#include <zypp/base/Gettext.h>
 
-#include "zypp/media/MediaCurl.h"
-#include "zypp/media/ProxyInfo.h"
-#include "zypp/media/MediaUserAuth.h"
-#include "zypp/media/CredentialManager.h"
-#include "zypp/media/CurlConfig.h"
-#include "zypp/thread/Once.h"
-#include "zypp/Target.h"
-#include "zypp/ZYppFactory.h"
-#include "zypp/ZConfig.h"
+#include <zypp/media/MediaCurl.h>
+#include <zypp/media/ProxyInfo.h>
+#include <zypp/media/MediaUserAuth.h>
+#include <zypp/media/CredentialManager.h>
+#include <zypp/media/CurlConfig.h>
+#include <zypp/media/CurlHelper.h>
+#include <zypp/Target.h>
+#include <zypp/ZYppFactory.h>
+#include <zypp/ZConfig.h>
 
 #include <cstdlib>
 #include <sys/types.h>
@@ -38,155 +38,13 @@
 #include <dirent.h>
 #include <unistd.h>
 
-#define  DETECT_DIR_INDEX       0
-#define  CONNECT_TIMEOUT        60
-#define  TRANSFER_TIMEOUT_MAX   60 * 60
+using std::endl;
 
-#define EXPLICITLY_NO_PROXY "_none_"
-
-#undef CURLVERSION_AT_LEAST
-#define CURLVERSION_AT_LEAST(M,N,O) LIBCURL_VERSION_NUM >= ((((M)<<8)+(N))<<8)+(O)
-
-using namespace std;
+using namespace internal;
 using namespace zypp::base;
 
-namespace
-{
-  zypp::thread::OnceFlag g_InitOnceFlag = PTHREAD_ONCE_INIT;
-  zypp::thread::OnceFlag g_FreeOnceFlag = PTHREAD_ONCE_INIT;
-
-  extern "C" void _do_free_once()
-  {
-    curl_global_cleanup();
-  }
-
-  extern "C" void globalFreeOnce()
-  {
-    zypp::thread::callOnce(g_FreeOnceFlag, _do_free_once);
-  }
-
-  extern "C" void _do_init_once()
-  {
-    CURLcode ret = curl_global_init( CURL_GLOBAL_ALL );
-    if ( ret != 0 )
-    {
-      WAR << "curl global init failed" << endl;
-    }
-
-    //
-    // register at exit handler ?
-    // this may cause trouble, because we can protect it
-    // against ourself only.
-    // if the app sets an atexit handler as well, it will
-    // cause a double free while the second of them runs.
-    //
-    //std::atexit( globalFreeOnce);
-  }
-
-  inline void globalInitOnce()
-  {
-    zypp::thread::callOnce(g_InitOnceFlag, _do_init_once);
-  }
-
-  int log_curl(CURL *curl, curl_infotype info,
-               char *ptr, size_t len, void *max_lvl)
-  {
-    std::string pfx(" ");
-    long        lvl = 0;
-    switch( info)
-    {
-      case CURLINFO_TEXT:       lvl = 1; pfx = "*"; break;
-      case CURLINFO_HEADER_IN:  lvl = 2; pfx = "<"; break;
-      case CURLINFO_HEADER_OUT: lvl = 2; pfx = ">"; break;
-      default:                                      break;
-    }
-    if( lvl > 0 && max_lvl != NULL && lvl <= *((long *)max_lvl))
-    {
-      std::string                            msg(ptr, len);
-      std::list<std::string>                 lines;
-      std::list<std::string>::const_iterator line;
-      zypp::str::split(msg, std::back_inserter(lines), "\r\n");
-      for(line = lines.begin(); line != lines.end(); ++line)
-      {
-        DBG << pfx << " " << *line << endl;
-      }
-    }
-    return 0;
-  }
-
-  static size_t log_redirects_curl( char *ptr, size_t size, size_t nmemb, void *userdata)
-  {
-    // INT << "got header: " << string(ptr, ptr + size*nmemb) << endl;
-
-    char * lstart = ptr, * lend = ptr;
-    size_t pos = 0;
-    size_t max = size * nmemb;
-    while (pos + 1 < max)
-    {
-      // get line
-      for (lstart = lend; *lend != '\n' && pos < max; ++lend, ++pos);
-
-      // look for "Location"
-      if ( lstart[0] == 'L'
-	&& lstart[1] == 'o'
-	&& lstart[2] == 'c'
-	&& lstart[3] == 'a'
-	&& lstart[4] == 't'
-	&& lstart[5] == 'i'
-	&& lstart[6] == 'o'
-	&& lstart[7] == 'n'
-	&& lstart[8] == ':' )
-      {
-	std::string line { lstart, *(lend-1)=='\r' ? lend-1 : lend };
-        DBG << "redirecting to " << line << endl;
-	if ( userdata ) {
-	  *reinterpret_cast<std::string *>( userdata ) = line;
-	}
-        return max;
-      }
-
-      // continue with the next line
-      if (pos + 1 < max)
-      {
-        ++lend;
-        ++pos;
-      }
-      else
-        break;
-    }
-
-    return max;
-  }
-}
-
 namespace zypp {
 
-  ///////////////////////////////////////////////////////////////////
-  namespace env
-  {
-    namespace
-    {
-      inline int getZYPP_MEDIA_CURL_IPRESOLVE()
-      {
-	int ret = 0;
-	if ( const char * envp = getenv( "ZYPP_MEDIA_CURL_IPRESOLVE" ) )
-	{
-	  WAR << "env set: $ZYPP_MEDIA_CURL_IPRESOLVE='" << envp << "'" << endl;
-	  if (      strcmp( envp, "4" ) == 0 )	ret = 4;
-	  else if ( strcmp( envp, "6" ) == 0 )	ret = 6;
-	}
-	return ret;
-      }
-    }
-
-    inline int ZYPP_MEDIA_CURL_IPRESOLVE()
-    {
-      static int _v = getZYPP_MEDIA_CURL_IPRESOLVE();
-      return _v;
-    }
-  } // namespace env
-  ///////////////////////////////////////////////////////////////////
-
   namespace media {
 
   namespace {
@@ -302,253 +160,10 @@ namespace zypp {
       // bytes uploaded at the moment the progress was last reported
       double                                        uload;
     };
-
-    ///////////////////////////////////////////////////////////////////
-
-    inline void escape( string & str_r,
-                        const char char_r, const string & escaped_r ) {
-      for ( string::size_type pos = str_r.find( char_r );
-            pos != string::npos; pos = str_r.find( char_r, pos ) ) {
-              str_r.replace( pos, 1, escaped_r );
-            }
-    }
-
-    inline string escapedPath( string path_r ) {
-      escape( path_r, ' ', "%20" );
-      return path_r;
-    }
-
-    inline string unEscape( string text_r ) {
-      char * tmp = curl_unescape( text_r.c_str(), 0 );
-      string ret( tmp );
-      curl_free( tmp );
-      return ret;
-    }
-
   }
 
-/**
- * Fills the settings structure using options passed on the url
- * for example ?timeout=x&proxy=foo
- */
-void fillSettingsFromUrl( const Url &url, TransferSettings &s )
-{
-    std::string param(url.getQueryParam("timeout"));
-    if( !param.empty())
-    {
-      long num = str::strtonum<long>(param);
-      if( num >= 0 && num <= TRANSFER_TIMEOUT_MAX)
-          s.setTimeout(num);
-    }
-
-    if ( ! url.getUsername().empty() )
-    {
-        s.setUsername(url.getUsername());
-        if ( url.getPassword().size() )
-            s.setPassword(url.getPassword());
-    }
-    else
-    {
-        // if there is no username, set anonymous auth
-        if ( ( url.getScheme() == "ftp" || url.getScheme() == "tftp" ) && s.username().empty() )
-            s.setAnonymousAuth();
-    }
-
-    if ( url.getScheme() == "https" )
-    {
-        s.setVerifyPeerEnabled(false);
-        s.setVerifyHostEnabled(false);
-
-        std::string verify( url.getQueryParam("ssl_verify"));
-        if( verify.empty() ||
-            verify == "yes")
-        {
-            s.setVerifyPeerEnabled(true);
-            s.setVerifyHostEnabled(true);
-        }
-        else if( verify == "no")
-        {
-            s.setVerifyPeerEnabled(false);
-            s.setVerifyHostEnabled(false);
-        }
-        else
-        {
-            std::vector<std::string>                 flags;
-            std::vector<std::string>::const_iterator flag;
-            str::split( verify, std::back_inserter(flags), ",");
-            for(flag = flags.begin(); flag != flags.end(); ++flag)
-            {
-                if( *flag == "host")
-                    s.setVerifyHostEnabled(true);
-                else if( *flag == "peer")
-                    s.setVerifyPeerEnabled(true);
-                else
-                    ZYPP_THROW(MediaBadUrlException(url, "Unknown ssl_verify flag"));
-            }
-        }
-    }
-
-    Pathname ca_path( url.getQueryParam("ssl_capath") );
-    if( ! ca_path.empty())
-    {
-        if( !PathInfo(ca_path).isDir() || ! ca_path.absolute())
-            ZYPP_THROW(MediaBadUrlException(url, "Invalid ssl_capath path"));
-        else
-            s.setCertificateAuthoritiesPath(ca_path);
-    }
-
-    Pathname client_cert( url.getQueryParam("ssl_clientcert") );
-    if( ! client_cert.empty())
-    {
-        if( !PathInfo(client_cert).isFile() || !client_cert.absolute())
-            ZYPP_THROW(MediaBadUrlException(url, "Invalid ssl_clientcert file"));
-        else
-            s.setClientCertificatePath(client_cert);
-    }
-    Pathname client_key( url.getQueryParam("ssl_clientkey") );
-    if( ! client_key.empty())
-    {
-        if( !PathInfo(client_key).isFile() || !client_key.absolute())
-            ZYPP_THROW(MediaBadUrlException(url, "Invalid ssl_clientkey file"));
-        else
-            s.setClientKeyPath(client_key);
-    }
-
-    param = url.getQueryParam( "proxy" );
-    if ( ! param.empty() )
-    {
-        if ( param == EXPLICITLY_NO_PROXY ) {
-	    // Workaround TransferSettings shortcoming: With an
-	    // empty proxy string, code will continue to look for
-	    // valid proxy settings. So set proxy to some non-empty
-	    // string, to indicate it has been explicitly disabled.
-	    s.setProxy(EXPLICITLY_NO_PROXY);
-            s.setProxyEnabled(false);
-        }
-        else {
-            string proxyport( url.getQueryParam( "proxyport" ) );
-            if ( ! proxyport.empty() ) {
-                param += ":" + proxyport;
-            }
-            s.setProxy(param);
-            s.setProxyEnabled(true);
-        }
-    }
-
-    param = url.getQueryParam( "proxyuser" );
-    if ( ! param.empty() )
-    {
-      s.setProxyUsername(param);
-      s.setProxyPassword(url.getQueryParam( "proxypass" ));
-    }
-
-    // HTTP authentication type
-    param = url.getQueryParam("auth");
-    if (!param.empty() && (url.getScheme() == "http" || url.getScheme() == "https"))
-    {
-        try
-        {
-	    CurlAuthData::auth_type_str2long(param);	// check if we know it
-        }
-        catch (MediaException & ex_r)
-	{
-	    DBG << "Rethrowing as MediaUnauthorizedException.";
-	    ZYPP_THROW(MediaUnauthorizedException(url, ex_r.msg(), "", ""));
-	}
-        s.setAuthType(param);
-    }
-
-    // workarounds
-    param = url.getQueryParam("head_requests");
-    if( !param.empty() && param == "no" )
-        s.setHeadRequestsAllowed(false);
-}
-
-/**
- * Reads the system proxy configuration and fills the settings
- * structure proxy information
- */
-void fillSettingsSystemProxy( const Url&url, TransferSettings &s )
-{
-    ProxyInfo proxy_info;
-    if ( proxy_info.useProxyFor( url ) )
-    {
-      // We must extract any 'user:pass' from the proxy url
-      // otherwise they won't make it into curl (.curlrc wins).
-      try {
-	Url u( proxy_info.proxy( url ) );
-	s.setProxy( u.asString( url::ViewOption::WITH_SCHEME + url::ViewOption::WITH_HOST + url::ViewOption::WITH_PORT ) );
-	// don't overwrite explicit auth settings
-	if ( s.proxyUsername().empty() )
-	{
-	  s.setProxyUsername( u.getUsername( url::E_ENCODED ) );
-	  s.setProxyPassword( u.getPassword( url::E_ENCODED ) );
-	}
-	s.setProxyEnabled( true );
-      }
-      catch (...) {}	// no proxy if URL is malformed
-    }
-}
-
 Pathname MediaCurl::_cookieFile = "/var/lib/YaST2/cookies";
 
-/**
- * initialized only once, this gets the anonymous id
- * from the target, which we pass in the http header
- */
-static const char *const anonymousIdHeader()
-{
-  // we need to add the release and identifier to the
-  // agent string.
-  // The target could be not initialized, and then this information
-  // is guessed.
-  static const std::string _value(
-      str::trim( str::form(
-          "X-ZYpp-AnonymousId: %s",
-          Target::anonymousUniqueId( Pathname()/*guess root*/ ).c_str() ) )
-  );
-  return _value.c_str();
-}
-
-/**
- * initialized only once, this gets the distribution flavor
- * from the target, which we pass in the http header
- */
-static const char *const distributionFlavorHeader()
-{
-  // we need to add the release and identifier to the
-  // agent string.
-  // The target could be not initialized, and then this information
-  // is guessed.
-  static const std::string _value(
-      str::trim( str::form(
-          "X-ZYpp-DistributionFlavor: %s",
-          Target::distributionFlavor( Pathname()/*guess root*/ ).c_str() ) )
-  );
-  return _value.c_str();
-}
-
-/**
- * initialized only once, this gets the agent string
- * which also includes the curl version
- */
-static const char *const agentString()
-{
-  // we need to add the release and identifier to the
-  // agent string.
-  // The target could be not initialized, and then this information
-  // is guessed.
-  static const std::string _value(
-    str::form(
-       "ZYpp %s (curl %s) %s"
-       , VERSION
-       , curl_version_info(CURLVERSION_NOW)->version
-       , Target::targetDistribution( Pathname()/*guess root*/ ).c_str()
-    )
-  );
-  return _value.c_str();
-}
-
 // we use this define to unbloat code as this C setting option
 // and catching exception is done frequently.
 /** \todo deprecate SET_OPTION and use the typed versions below. */
@@ -576,7 +191,7 @@ MediaCurl::MediaCurl( const Url &      url_r,
 
   MIL << "MediaCurl::MediaCurl(" << url_r << ", " << attach_point_hint_r << ")" << endl;
 
-  globalInitOnce();
+  globalInitCurlOnce();
 
   if( !attachPoint().empty())
   {
@@ -601,27 +216,7 @@ MediaCurl::MediaCurl( const Url &      url_r,
 
 Url MediaCurl::clearQueryString(const Url &url) const
 {
-  Url curlUrl (url);
-  curlUrl.setUsername( "" );
-  curlUrl.setPassword( "" );
-  curlUrl.setPathParams( "" );
-  curlUrl.setFragment( "" );
-  curlUrl.delQueryParam("cookies");
-  curlUrl.delQueryParam("proxy");
-  curlUrl.delQueryParam("proxyport");
-  curlUrl.delQueryParam("proxyuser");
-  curlUrl.delQueryParam("proxypass");
-  curlUrl.delQueryParam("ssl_capath");
-  curlUrl.delQueryParam("ssl_verify");
-  curlUrl.delQueryParam("ssl_clientcert");
-  curlUrl.delQueryParam("timeout");
-  curlUrl.delQueryParam("auth");
-  curlUrl.delQueryParam("username");
-  curlUrl.delQueryParam("password");
-  curlUrl.delQueryParam("mediahandler");
-  curlUrl.delQueryParam("credentials");
-  curlUrl.delQueryParam("head_requests");
-  return curlUrl;
+  return internal::clearQueryString(url);
 }
 
 TransferSettings & MediaCurl::settings()
@@ -795,7 +390,7 @@ void MediaCurl::setupEasy()
   if ( _settings.userPassword().size() )
   {
     SET_OPTION(CURLOPT_USERPWD, _settings.userPassword().c_str());
-    string use_auth = _settings.authType();
+    std::string use_auth = _settings.authType();
     if (use_auth.empty())
       use_auth = "digest,basic";	// our default
     long auth = CurlAuthData::auth_type_str2long(use_auth);
@@ -819,7 +414,7 @@ void MediaCurl::setupEasy()
      *  If not provided, $HOME/.curlrc is evaluated
      *---------------------------------------------------------------*/
 
-    string proxyuserpwd = _settings.proxyUserPassword();
+    std::string proxyuserpwd = _settings.proxyUserPassword();
 
     if ( proxyuserpwd.empty() )
     {
@@ -840,7 +435,7 @@ void MediaCurl::setupEasy()
 
     if ( ! proxyuserpwd.empty() )
     {
-      SET_OPTION(CURLOPT_PROXYUSERPWD, unEscape( proxyuserpwd ).c_str());
+      SET_OPTION(CURLOPT_PROXYUSERPWD, curlUnEscape( proxyuserpwd ).c_str());
     }
   }
 #if CURLVERSION_AT_LEAST(7,19,4)
@@ -875,6 +470,7 @@ void MediaCurl::setupEasy()
    *---------------------------------------------------------------*/
 
   _currentCookieFile = _cookieFile.asString();
+  filesystem::assert_file_mode( _currentCookieFile, 0600 );
   if ( str::strToBool( _url.getQueryParam( "cookies" ), true ) )
     SET_OPTION(CURLOPT_COOKIEFILE, _currentCookieFile.c_str() );
   else
@@ -1112,12 +708,12 @@ void MediaCurl::evaluateCurlCode(const Pathname &filename,
                                               &httpReturnCode );
         if ( infoRet == CURLE_OK )
         {
-          string msg = "HTTP response: " + str::numstring( httpReturnCode );
+          std::string msg = "HTTP response: " + str::numstring( httpReturnCode );
           switch ( httpReturnCode )
           {
           case 401:
           {
-            string auth_hint = getAuthHint();
+            std::string auth_hint = getAuthHint();
 
             DBG << msg << " Login failed (URL: " << url.asString() << ")" << std::endl;
             DBG << "MediaUnauthorizedException auth hint: '" << auth_hint << "'" << std::endl;
@@ -1134,10 +730,10 @@ void MediaCurl::evaluateCurlCode(const Pathname &filename,
             ZYPP_THROW(MediaTimeoutException(url));
           case 403:
           {
-            string msg403;
-	    if ( url.getHost().find(".suse.com") != string::npos )
+            std::string msg403;
+	    if ( url.getHost().find(".suse.com") != std::string::npos )
 	      msg403 = _("Visit the SUSE Customer Center to check whether your registration is valid and has not expired.");
-	    else if (url.asString().find("novell.com") != string::npos)
+	    else if (url.asString().find("novell.com") != std::string::npos)
               msg403 = _("Visit the Novell Customer Center to check whether your registration is valid and has not expired.");
             ZYPP_THROW(MediaForbiddenException(url, msg403));
           }
@@ -1151,7 +747,7 @@ void MediaCurl::evaluateCurlCode(const Pathname &filename,
         }
         else
         {
-          string msg = "Unable to retrieve HTTP response:";
+          std::string msg = "Unable to retrieve HTTP response:";
           DBG << msg << " (URL: " << url.asString() << ")" << std::endl;
           ZYPP_THROW(MediaCurlException(url, msg, _curlError));
         }
@@ -1240,7 +836,7 @@ bool MediaCurl::doGetDoesFileExist( const Pathname & filename ) const
     // contains an absolute path.
   //
   _lastRedirect.clear();
-  string urlBuffer( curlUrl.asString());
+  std::string urlBuffer( curlUrl.asString());
   CURLcode ret = curl_easy_setopt( _curl, CURLOPT_URL,
                                    urlBuffer.c_str() );
   if ( ret != 0 ) {
@@ -1270,7 +866,7 @@ bool MediaCurl::doGetDoesFileExist( const Pathname & filename ) const
     ZYPP_THROW(MediaCurlSetOptException(url, _curlError));
   }
 
-  FILE *file = ::fopen( "/dev/null", "w" );
+  AutoFILE file { ::fopen( "/dev/null", "w" ) };
   if ( !file ) {
       ERR << "fopen failed for /dev/null" << endl;
       curl_easy_setopt( _curl, CURLOPT_NOBODY, 0L);
@@ -1287,9 +883,8 @@ bool MediaCurl::doGetDoesFileExist( const Pathname & filename ) const
       ZYPP_THROW(MediaWriteException("/dev/null"));
   }
 
-  ret = curl_easy_setopt( _curl, CURLOPT_WRITEDATA, file );
+  ret = curl_easy_setopt( _curl, CURLOPT_WRITEDATA, (*file) );
   if ( ret != 0 ) {
-      ::fclose(file);
       std::string err( _curlError);
       curl_easy_setopt( _curl, CURLOPT_RANGE, NULL );
       curl_easy_setopt( _curl, CURLOPT_NOBODY, 0L);
@@ -1336,10 +931,6 @@ bool MediaCurl::doGetDoesFileExist( const Pathname & filename ) const
     }
   }
 
-  // if the code is not zero, close the file
-  if ( ok != 0 )
-      ::fclose(file);
-
   // as we are not having user interaction, the user can't cancel
   // the file existence checking, a callback or timeout return code
   // will be always a timeout.
@@ -1409,34 +1000,34 @@ void MediaCurl::doGetFileCopy(const Pathname & filename , const Pathname & targe
     if( assert_dir( dest.dirname() ) )
     {
       DBG << "assert_dir " << dest.dirname() << " failed" << endl;
-      Url url(getFileUrl(filename));
-      ZYPP_THROW( MediaSystemException(url, "System error on " + dest.dirname().asString()) );
-    }
-    string destNew = target.asString() + ".new.zypp.XXXXXX";
-    char *buf = ::strdup( destNew.c_str());
-    if( !buf)
-    {
-      ERR << "out of memory for temp file name" << endl;
-      Url url(getFileUrl(filename));
-      ZYPP_THROW(MediaSystemException(url, "out of memory for temp file name"));
+      ZYPP_THROW( MediaSystemException(getFileUrl(filename), "System error on " + dest.dirname().asString()) );
     }
 
-    int tmp_fd = ::mkostemp( buf, O_CLOEXEC );
-    if( tmp_fd == -1)
+    ManagedFile destNew { target.extend( ".new.zypp.XXXXXX" ) };
+    AutoFILE file;
     {
-      free( buf);
-      ERR << "mkstemp failed for file '" << destNew << "'" << endl;
-      ZYPP_THROW(MediaWriteException(destNew));
-    }
-    destNew = buf;
-    free( buf);
+      AutoFREE<char> buf { ::strdup( (*destNew).c_str() ) };
+      if( ! buf )
+      {
+	ERR << "out of memory for temp file name" << endl;
+	ZYPP_THROW(MediaSystemException(getFileUrl(filename), "out of memory for temp file name"));
+      }
 
-    FILE *file = ::fdopen( tmp_fd, "we" );
-    if ( !file ) {
-      ::close( tmp_fd);
-      filesystem::unlink( destNew );
-      ERR << "fopen failed for file '" << destNew << "'" << endl;
-      ZYPP_THROW(MediaWriteException(destNew));
+      AutoFD tmp_fd { ::mkostemp( buf, O_CLOEXEC ) };
+      if( tmp_fd == -1 )
+      {
+	ERR << "mkstemp failed for file '" << destNew << "'" << endl;
+	ZYPP_THROW(MediaWriteException(destNew));
+      }
+      destNew = ManagedFile( (*buf), filesystem::unlink );
+
+      file = ::fdopen( tmp_fd, "we" );
+      if ( ! file )
+      {
+	ERR << "fopen failed for file '" << destNew << "'" << endl;
+	ZYPP_THROW(MediaWriteException(destNew));
+      }
+      tmp_fd.resetDispose();	// don't close it here! ::fdopen moved ownership to file
     }
 
     DBG << "dest: " << dest << endl;
@@ -1459,8 +1050,6 @@ void MediaCurl::doGetFileCopy(const Pathname & filename , const Pathname & targe
     }
     catch (Exception &e)
     {
-      ::fclose( file );
-      filesystem::unlink( destNew );
       curl_easy_setopt(_curl, CURLOPT_TIMECONDITION, CURL_TIMECOND_NONE);
       curl_easy_setopt(_curl, CURLOPT_TIMEVALUE, 0L);
       ZYPP_RETHROW(e);
@@ -1494,22 +1083,20 @@ void MediaCurl::doGetFileCopy(const Pathname & filename , const Pathname & targe
       {
         ERR << "Failed to chmod file " << destNew << endl;
       }
-      if (::fclose( file ))
+
+      file.resetDispose();	// we're going to close it manually here
+      if ( ::fclose( file ) )
       {
         ERR << "Fclose failed for file '" << destNew << "'" << endl;
         ZYPP_THROW(MediaWriteException(destNew));
       }
+
       // move the temp file into dest
       if ( rename( destNew, dest ) != 0 ) {
         ERR << "Rename failed" << endl;
         ZYPP_THROW(MediaWriteException(dest));
       }
-    }
-    else
-    {
-      // close and remove the temp file
-      ::fclose( file );
-      filesystem::unlink( destNew );
+      destNew.resetDispose();	// no more need to unlink it
     }
 
     DBG << "done: " << PathInfo(dest) << endl;
@@ -1545,7 +1132,7 @@ void MediaCurl::doGetFileCopyFile(const Pathname & filename , const Pathname & d
     // contains an absolute path.
     //
     _lastRedirect.clear();
-    string urlBuffer( curlUrl.asString());
+    std::string urlBuffer( curlUrl.asString());
     CURLcode ret = curl_easy_setopt( _curl, CURLOPT_URL,
                                      urlBuffer.c_str() );
     if ( ret != 0 ) {
@@ -1711,7 +1298,7 @@ CURL *MediaCurl::progressCallback_getcurl( void *clientp )
 
 ///////////////////////////////////////////////////////////////////
 
-string MediaCurl::getAuthHint() const
+std::string MediaCurl::getAuthHint() const
 {
   long auth_info = CURLAUTH_NONE;
 
@@ -1740,7 +1327,7 @@ void MediaCurl::resetExpectedFileSize(void *clientp, const ByteCount &expectedFi
 
 ///////////////////////////////////////////////////////////////////
 
-bool MediaCurl::authenticate(const string & availAuthTypes, bool firstTry) const
+bool MediaCurl::authenticate(const std::string & availAuthTypes, bool firstTry) const
 {
   //! \todo need a way to pass different CredManagerOptions here
   CredentialManager cm(CredManagerOptions(ZConfig::instance().repoManagerRoot()));
@@ -1772,7 +1359,7 @@ bool MediaCurl::authenticate(const string & availAuthTypes, bool firstTry) const
     // indicate we have no good credentials from CM
     cmcred.reset();
 
-    string prompt_msg = str::Format(_("Authentication required for '%s'")) % _url.asString();
+    std::string prompt_msg = str::Format(_("Authentication required for '%s'")) % _url.asString();
 
     // set available authentication types from the exception
     // might be needed in prompt
@@ -1841,6 +1428,9 @@ bool MediaCurl::authenticate(const string & availAuthTypes, bool firstTry) const
   return false;
 }
 
+//need a out of line definiton, otherwise vtable is emitted for every translation unit
+MediaCurl::Callbacks::~Callbacks() {}
+
 
   } // namespace media
 } // namespace zypp
