@@ -13,26 +13,26 @@
 #include <fstream>
 #include <boost/mpl/int.hpp>
 
-#include "zypp/base/Easy.h"
-#include "zypp/base/LogTools.h"
-#include "zypp/base/Gettext.h"
-#include "zypp/base/Exception.h"
-#include "zypp/base/Measure.h"
-#include "zypp/base/WatchFile.h"
-#include "zypp/base/Sysconfig.h"
-#include "zypp/base/IOStream.h"
+#include <zypp/base/Easy.h>
+#include <zypp/base/LogTools.h>
+#include <zypp/base/Gettext.h>
+#include <zypp/base/Exception.h>
+#include <zypp/base/Measure.h>
+#include <zypp/base/WatchFile.h>
+#include <zypp/base/Sysconfig.h>
+#include <zypp/base/IOStream.h>
 
-#include "zypp/ZConfig.h"
+#include <zypp/ZConfig.h>
 
-#include "zypp/sat/detail/PoolImpl.h"
-#include "zypp/sat/SolvableSet.h"
-#include "zypp/sat/Pool.h"
-#include "zypp/Capability.h"
-#include "zypp/Locale.h"
-#include "zypp/PoolItem.h"
+#include <zypp/sat/detail/PoolImpl.h>
+#include <zypp/sat/SolvableSet.h>
+#include <zypp/sat/Pool.h>
+#include <zypp/Capability.h>
+#include <zypp/Locale.h>
+#include <zypp/PoolItem.h>
 
-#include "zypp/target/modalias/Modalias.h"
-#include "zypp/media/MediaPriority.h"
+#include <zypp/target/modalias/Modalias.h>
+#include <zypp/media/MediaPriority.h>
 
 extern "C"
 {
@@ -522,10 +522,10 @@ namespace zypp
 	    { localeIds.removed().insert( IdString(lang) ); }
 	  }
 
-	  // Assert that TrackedLocaleIds::current is not empty.
-	  // If, so fill in LanguageCode::enCode as last resort.
-	  if ( localeIds.current().empty() )
-	  { localeIds.current().insert( IdString(Locale::enCode) ); }
+	  // bsc#1155678: We try to differ between an empty RequestedLocales
+	  // and one containing 'en' (explicit or as fallback). An empty RequestedLocales
+	  // should not even drag in recommended 'en' packages. So we no longer enforce
+	  // 'en' being in the set.
 	}
 	return *_trackedLocaleIdsPtr;
       }
