@@ -22,25 +22,18 @@
 
 #define ZYPP_USE_RESOLVER_INTERNALS
 
-#include "zypp/solver/detail/Resolver.h"
-#include "zypp/solver/detail/Testcase.h"
-#include "zypp/solver/detail/SATResolver.h"
-#include "zypp/solver/detail/ItemCapKind.h"
-#include "zypp/solver/detail/SolutionAction.h"
-#include "zypp/solver/detail/SolverQueueItem.h"
+#include <zypp/base/LogTools.h>
+#include <zypp/base/Algorithm.h>
 
-#include "zypp/Capabilities.h"
-#include "zypp/ZConfig.h"
-#include "zypp/base/Logger.h"
-#include "zypp/base/String.h"
-#include "zypp/base/Gettext.h"
-#include "zypp/base/Algorithm.h"
-#include "zypp/ResPool.h"
-#include "zypp/ResFilters.h"
-#include "zypp/sat/Pool.h"
-#include "zypp/sat/Solvable.h"
-#include "zypp/sat/Transaction.h"
-#include "zypp/ResolverProblem.h"
+#include <zypp/solver/detail/Resolver.h>
+#include <zypp/solver/detail/Testcase.h>
+#include <zypp/solver/detail/SATResolver.h>
+#include <zypp/solver/detail/ItemCapKind.h>
+#include <zypp/solver/detail/SolutionAction.h>
+#include <zypp/solver/detail/SolverQueueItem.h>
+
+#include <zypp/ZConfig.h>
+#include <zypp/sat/Transaction.h>
 
 #define MAXSOLVERRUNS 5
 
@@ -60,7 +53,6 @@ namespace zypp
     namespace detail
     { ///////////////////////////////////////////////////////////////////
 
-      //using namespace std;
 
 //---------------------------------------------------------------------------
 
@@ -107,6 +99,10 @@ Resolver::~Resolver()
 
 //---------------------------------------------------------------------------
 // forward flags too SATResolver
+
+void Resolver::setFocus( ResolverFocus focus_r )	{ _satResolver->_focus = ( focus_r == ResolverFocus::Default ) ? ZConfig::instance().solver_focus() : focus_r; }
+ResolverFocus Resolver::focus() const			{ return _satResolver->_focus; }
+
 #define ZOLV_FLAG_TRIBOOL( ZSETTER, ZGETTER, ZVARNAME, ZVARDEFAULT )			\
     void Resolver::ZSETTER( TriBool state_r )						\
     { _satResolver->ZVARNAME = indeterminate(state_r) ? ZVARDEFAULT : bool(state_r); }	\
