@@ -19,47 +19,81 @@
 #include <sys/types.h>
 #include <dirent.h>
 
-#include "zypp/base/LogTools.h"
-#include "zypp/base/Exception.h"
-#include "zypp/base/Iterator.h"
-#include "zypp/base/Gettext.h"
-#include "zypp/base/IOStream.h"
-#include "zypp/base/Functional.h"
-#include "zypp/base/UserRequestException.h"
-#include "zypp/base/Json.h"
+#include <zypp/base/LogTools.h>
+#include <zypp/base/Exception.h>
+#include <zypp/base/Iterator.h>
+#include <zypp/base/Gettext.h>
+#include <zypp/base/IOStream.h>
+#include <zypp/base/Functional.h>
+#include <zypp/base/UserRequestException.h>
+#include <zypp/base/Json.h>
 
-#include "zypp/ZConfig.h"
-#include "zypp/ZYppFactory.h"
-#include "zypp/PathInfo.h"
+#include <zypp/ZConfig.h>
+#include <zypp/ZYppFactory.h>
+#include <zypp/PathInfo.h>
 
-#include "zypp/PoolItem.h"
-#include "zypp/ResObjects.h"
-#include "zypp/Url.h"
-#include "zypp/TmpPath.h"
-#include "zypp/RepoStatus.h"
-#include "zypp/ExternalProgram.h"
-#include "zypp/Repository.h"
-#include "zypp/ShutdownLock_p.h"
+#include <zypp/PoolItem.h>
+#include <zypp/ResObjects.h>
+#include <zypp/Url.h>
+#include <zypp/TmpPath.h>
+#include <zypp/RepoStatus.h>
+#include <zypp/ExternalProgram.h>
+#include <zypp/Repository.h>
+#include <zypp/ShutdownLock_p.h>
 
-#include "zypp/ResFilters.h"
-#include "zypp/HistoryLog.h"
-#include "zypp/target/TargetImpl.h"
-#include "zypp/target/TargetCallbackReceiver.h"
-#include "zypp/target/rpm/librpmDb.h"
-#include "zypp/target/CommitPackageCache.h"
-#include "zypp/target/RpmPostTransCollector.h"
+#include <zypp/ResFilters.h>
+#include <zypp/HistoryLog.h>
+#include <zypp/target/TargetImpl.h>
+#include <zypp/target/TargetCallbackReceiver.h>
+#include <zypp/target/rpm/librpmDb.h>
+#include <zypp/target/CommitPackageCache.h>
+#include <zypp/target/RpmPostTransCollector.h>
 
-#include "zypp/parser/ProductFileReader.h"
-#include "zypp/repo/SrcPackageProvider.h"
+#include <zypp/parser/ProductFileReader.h>
+#include <zypp/repo/SrcPackageProvider.h>
 
-#include "zypp/sat/Pool.h"
-#include "zypp/sat/detail/PoolImpl.h"
-#include "zypp/sat/SolvableSpec.h"
-#include "zypp/sat/Transaction.h"
+#include <zypp/sat/Pool.h>
+#include <zypp/sat/detail/PoolImpl.h>
+#include <zypp/sat/SolvableSpec.h>
+#include <zypp/sat/Transaction.h>
 
-#include "zypp/PluginExecutor.h"
+#include <zypp/PluginExecutor.h>
 
-using namespace std;
+using std::endl;
+
+///////////////////////////////////////////////////////////////////
+extern "C"
+{
+#include <solv/repo_rpmdb.h>
+}
+namespace zypp
+{
+  namespace target
+  {
+    inline std::string rpmDbStateHash( const Pathname & root_r )
+    {
+      std::string ret;
+      AutoDispose<void*> state { ::rpm_state_create( sat::Pool::instance().get(), root_r.c_str() ), ::rpm_state_free };
+      AutoDispose<Chksum*> chk { ::solv_chksum_create( REPOKEY_TYPE_SHA1 ), []( Chksum *chk ) -> void {
+	::solv_chksum_free( chk, nullptr );
+      } };
+      if ( ::rpm_hash_database_state( state, chk ) == 0 )
+      {
+	int md5l;
+	const unsigned char * md5 = ::solv_chksum_get( chk, &md5l );
+	ret = ::pool_bin2hex( sat::Pool::instance().get(), md5, md5l );
+      }
+      else
+	WAR << "rpm_hash_database_state failed" << endl;
+      return ret;
+    }
+
+    inline RepoStatus rpmDbRepoStatus( const Pathname & root_r )
+    { return RepoStatus( rpmDbStateHash( root_r ), Date() ); }
+
+  } // namespace target
+} // namespace
+///////////////////////////////////////////////////////////////////
 
 ///////////////////////////////////////////////////////////////////
 namespace zypp
@@ -571,7 +605,7 @@ namespace zypp
         else if ( format == DIGEST || format == BULK )
         {
           filesystem::TmpFile tmpfile;
-          ofstream out( tmpfile.path().c_str() );
+	  std::ofstream out( tmpfile.path().c_str() );
           for_( it, notifications_r.begin(), notifications_r.end() )
           {
             if ( format == DIGEST )
@@ -664,6 +698,31 @@ namespace zypp
         sendNotification( root_r, result_r.updateMessages() );
       }
 
+      /** jsc#SLE-5116: Log patch status changes to history.
+       * Adjust precomputed set if transaction is incomplete.
+       */
+      void logPatchStatusChanges( const sat::Transaction & transaction_r, TargetImpl & target_r )
+      {
+	ResPool::ChangedPseudoInstalled changedPseudoInstalled { ResPool::instance().changedPseudoInstalled() };
+	if ( changedPseudoInstalled.empty() )
+	  return;
+
+	if ( ! transaction_r.actionEmpty( ~sat::Transaction::STEP_DONE ) )
+	{
+	  // Need to recompute the patch list if commit is incomplete!
+	  // We remember the initially established status, then reload the
+	  // Target to get the current patch status. Then compare.
+	  WAR << "Need to recompute the patch status changes as commit is incomplete!" << endl;
+	  ResPool::EstablishedStates establishedStates{ ResPool::instance().establishedStates() };
+	  target_r.load();
+	  changedPseudoInstalled = establishedStates.changedPseudoInstalled();
+	}
+
+	HistoryLog historylog;
+	for ( const auto & el : changedPseudoInstalled )
+	  historylog.patchStateChange( el.first, el.second );
+      }
+
       /////////////////////////////////////////////////////////////////
     } // namespace
     ///////////////////////////////////////////////////////////////////
@@ -689,7 +748,7 @@ namespace zypp
     , _autoInstalledFile( home() / "AutoInstalled" )
     , _hardLocksFile( Pathname::assertprefix( _root, ZConfig::instance().locksFile() ) )
     {
-      _rpm.initDatabase( root_r, Pathname(), doRebuild_r );
+      _rpm.initDatabase( root_r, doRebuild_r );
 
       HistoryLog::setRoot(_root);
 
@@ -714,9 +773,9 @@ namespace zypp
      */
     void updateFileContent( const Pathname &filename,
                             boost::function<bool ()> condition,
-                            boost::function<string ()> value )
+			    boost::function<std::string ()> value )
     {
-        string val = value();
+        std::string val = value();
         // if the value is empty, then just dont
         // do anything, regardless of the condition
         if ( val.empty() )
@@ -788,7 +847,7 @@ namespace zypp
           return;
       }
 
-      string flavor = p->flavor();
+      std::string flavor = p->flavor();
 
       try
       {
@@ -796,7 +855,7 @@ namespace zypp
         updateFileContent( flavorpath,
                            // only if flavor is not empty
                            functor::Constant<bool>( ! flavor.empty() ),
-                           functor::Constant<string>(flavor) );
+			   functor::Constant<std::string>(flavor) );
       }
       catch ( const Exception &e )
       {
@@ -843,7 +902,7 @@ namespace zypp
       bool build_rpm_solv = true;
       // lets see if the rpm solv cache exists
 
-      RepoStatus rpmstatus( RepoStatus(_root/"var/lib/rpm/Name") && RepoStatus(_root/"etc/products.d") );
+      RepoStatus rpmstatus( rpmDbRepoStatus(_root) && RepoStatus(_root/"etc/products.d") );
 
       bool solvexisted = PathInfo(rpmsolv).isExist();
       if ( solvexisted )
@@ -1127,6 +1186,8 @@ namespace zypp
     {
       // ----------------------------------------------------------------- //
       ZYppCommitPolicy policy_r( policy_rX );
+      bool explicitDryRun = policy_r.dryRun();	// explicit dry run will trigger a fileconflict check, implicit (download-only) not.
+
       ShutdownLock lck("Zypp commit running.");
 
       // Fake outstanding YCP fix: Honour restriction to media 1
@@ -1346,12 +1407,20 @@ namespace zypp
 	  else
 	  {
 	    DBG << "dryRun/downloadOnly: Not installing/deleting anything." << endl;
+	    if ( explicitDryRun ) {
+	      // if cache is preloaded, check for file conflicts
+	      commitFindFileConflicts( policy_r, result );
+	    }
 	  }
 	}
       }
       else
       {
         DBG << "dryRun: Not downloading/installing/deleting anything." << endl;
+	if ( explicitDryRun ) {
+	  // if cache is preloaded, check for file conflicts
+	  commitFindFileConflicts( policy_r, result );
+	}
       }
 
       ///////////////////////////////////////////////////////////////////
@@ -1615,11 +1684,18 @@ namespace zypp
                 }
                 else
                 {
-                  PathInfo referenceFile( Pathname::assertprefix( _root, Pathname( "/etc/products.d" ) ) / referenceFilename );
-                  if ( ! referenceFile.isFile() || filesystem::unlink( referenceFile.path() ) != 0 )
-                  {
-                    ERR << "Delete orphan product failed: " << referenceFile << endl;
-                  }
+		  Pathname referencePath { Pathname("/etc/products.d") / referenceFilename };	// no root prefix for rpmdb lookup!
+		  if ( ! rpm().hasFile( referencePath.asString() ) )
+		  {
+		    // If it's not owned by a package, we can delete it.
+		    referencePath = Pathname::assertprefix( _root, referencePath );	// now add a root prefix
+		    if ( filesystem::unlink( referencePath ) != 0 )
+		      ERR << "Delete orphan product failed: " << referencePath << endl;
+		  }
+		  else
+		  {
+		    WAR << "Won't remove orphan product: '/etc/products.d/" << referenceFilename << "' is owned by a package." << endl;
+		  }
                 }
               }
             }
@@ -1660,8 +1736,14 @@ namespace zypp
 			   result_r );
       }
 
+      // jsc#SLE-5116: Log patch status changes to history
+      // NOTE: Should be the last action as it may need to reload
+      // the Target in case of an incomplete transaction.
+      logPatchStatusChanges( result_r.transaction(), *this );
+
       if ( abort )
       {
+	HistoryLog().comment( "Commit was aborted." );
         ZYPP_THROW( TargetAbortedException( ) );
       }
     }
@@ -1678,12 +1760,6 @@ namespace zypp
       return _rpm.hasFile(path_str, name_str);
     }
 
-
-    Date TargetImpl::timestamp() const
-    {
-      return _rpm.timestamp();
-    }
-
     ///////////////////////////////////////////////////////////////////
     namespace
     {
